;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; boot.asm   v0.00.01   September 26th, 1998
;; NASM Assembly
;;
;; Part of ParticleOS booting procedure.
;; (09/26/1998,09/27/1998,09/28/1998,09/29/1998,2/15/1999,3/1/2000
;;  3/3/2000, 3/7/2000, 8/14/2001, 11/28/2001, 01/20/2002)
;;
;; Parts of this code are based on the Linux booter.
;;
;; (C) 1998-2002, Particle
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; when the machine starts up, we have:
;; F0000h - FFFFFh ROM BIOS control basic hardware, etc.
;; D0000h - EFFFFh free for additional BIOS extensions (usually)
;; C0000h - CFFFFh for BIOS extensions (disk controllers, etc.)
;; A0000h - BFFFFh for video memory
;; 00600h - 9FFFFh for user code
;; 00400h - 005FFh for bios data area
;; 00000h - 003FFh for interrupt table
;;
;; usable: A0000h - 00600h = 9FA00h = 653824 bytes
;; addressable: FFFFFh = 1,048,575 bytes
;; max segment length: FFFFh = 65535 bytes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; when the machine starts up, BIOS puts first sector
;; (512 <200h> bytes) of  booting media into location 07C00h.

; Make DS point to our program; When the machine boots, BIOS
; puts our program (first sector of bootdisk) at: segment
; 0x07C0, it is our job to make sure we load the rest of it
; from there.

        mov     ax,0x07C0       ; where bios loaded out program
        mov     ds,ax

; move the system (this program) out of the way to an
; arbitrary location, 0x90000 (gives us 64k to work with ;-)

        mov     ax,0x9000
        mov     es,ax           ; set dest segment
        mov     cx,80h          ; move 512 bytes
        xor     si,si           ; clear pointer
        xor     di,di           ; clear pointer
        cld                     ; clear direction flag
        rep     movsd           ; move dwords at a time
        jmp     0x9000:start    ; jump to the new start...
start:

; init all the segments; so that the system memory referencing
; and stack works

        mov     ds,ax           ; AX still contains 0x9000
        mov     ss,ax           ; use the same seg for stack
        mov     di,0xFFFF-12    ; This is where we will copy the floppy param block
        mov     sp,di           ; give us as much stack as possible

        mov     bx,loadingmsg
        call    printstr             ; print message

; to allow for multisector reading of floopy disks, we need to
; change the IVT (Interrupt Vector Table) to point to our own
; Floppy Disk Parameter Block. The vector 0x1E, contains address
; to this table. We need to copy this table from there, modify
; it to allow for multi sector reads, and reset the IVT to point
; to the new one. (so that its referenced when we use BIOS
; system calls for reading)
;
; Format of the table Floppy Disk Parameter Block follows.
; offset        contents
; 00            bits 4-7: step rate, 0-3: head unload time
; 01            bits 0-6: head load time, 7: DMA flag
; 02            motor off time
; 03            sector size in bytes (ms-dos standard: 2)
; 04            last sector on track (!WHAT WE NEED HERE!)
; 05            size of gap between sectors on read/write
; 06            length of data transfer (standard: 255)
; 07            size of gap between sectors on format
; 08            byte for sector filling on format
; 09            head settle time in ms
; 10            motor start-up time
;
; We need to set the last sector on track to some large
; number; Linux sets it to 36, that's the choice we'll have here.

        ; go to the 0000:0078 and get address of current
        ; Floppy Disk Parameter Block.

        mov     fs,cx           ; CX is still 0
        mov     bx,0x78

        ; there is a way to do this without using FS, but with
        ; ES, but it requires more code (pushing, poping, etc.)
        ; (I've tried it, and it doesn't look nice.)

        push    ds              ; save DS

        lds     si,[fs:bx]      ; load DS:SI with address from [fs:bx]
        push    di              ; save DI for later (location of new floppy params)

        cld                     ; clear direction flag (just in case)
        mov     cx,3            ; 12 bytes to copy
        rep     movsd           ; copy DWORD at a time (3 * 4 = 12)

        ; notice that we're copying 12 bytes, when the Floppy
        ; Disk Parameter Block is only 11... we're WASTING a
        ; whole BYTE!!!  ;-)

        pop     di              ; restore DI back to floppy disk params
        pop     ds              ; restore DS back to INITSEG
        mov byte [di+4],36       ; setup our value (enough for ED 2.88 floppy)

        ; setup the IVT table to point to our new Disk
        ; Parameter Block

        mov     [fs:bx],di
        mov     [fs:bx + 2],es

        ; Now, move the setup program (from disk) to memory
        ; right after the boot program. (this one starts at
        ; sector 2 (and we move it after 0x200, which is 512
        ; byte size of our boot program)...

load_setup:
        ;; cx is already 0

        ; reset disk system, AH = 00h, DL = DRIVE, INT 13h
        xor     ah,ah
        xor     dl,dl
        int     0x13
        ; we *should* check for an error here, but Linux doesn't do it, so we won't either.

        xor     dx,dx           ; drive 0, head 0
        mov     cl,0x02         ; sector 2, track 0
        mov     bx,0x0200       ; address 512
        mov     ah,0x02         ; function 2 of int 13h (read)
        mov     al,[setup_sectors] ; read that many sectors
        int     0x13            ; call BIOS
        jnc     load_setup.loaded ; everything is ok

        ;; PRINT some ERROR info if there was an error
        push    bx
        mov     bx,diskerror
        call    printstr
        pop     bx

        ; try again (until someone reboots)
        jmp     load_setup

.loaded:

        ; setup is now loaded

        ; jump to setup

        jmp 0x9020:0

%include "printstr.inc"

; our data
diskerror       db      10,13,'disk error',0
loadingmsg      db      13,10,'Hello World: boot.asm',0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; go to this location in assembly
;;; '$' stands for current location in assembly, '$$' represents
;;; current offset in the current section. thus, $-$$ is relative
;;; position in the section... thus, if say we're at 200 bytes
;;; of code, then we're filling the rest till 510 with 0...

TIMES 511-($-$$) DB 0
setup_sectors   db   0

