/*
 * build.c
 *
 * Copyright(c) 2002, Particle
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

/**
 * prints out the message, and exits with error code 1
 */
void pError(char *error, ...)
{
    va_list argptr;
    printf ("error: ");
    va_start (argptr,error);
    vprintf (error,argptr);
    va_end (argptr);
    printf ("\n");
    exit (1);
}

/**
 * return the length of the file
 */
int fileLength(FILE *f){
    int pos,end;
    if(f == (FILE*)0)
        return -1;
    pos = ftell(f);
    fseek(f,0,SEEK_END);
    end = ftell(f);
    fseek(f,pos,SEEK_SET);
    return end;
}

/**
 * open file for writing
 */
FILE* fileOpenWrite(char *filename){
    FILE* f;
    f = fopen(filename,"wb");
    return f;
}

/**
 * open file for writing
 * safe version
 */
FILE* fileOpenWriteSafe(char *filename){
    FILE* f;
    f = fileOpenWrite(filename);
    if(!f)
        pError("can't open file for writing: %s",filename);
    return f;
}

/**
 * open file for reading
 */
FILE* fileOpenRead(char *filename){
    FILE* f;
    f = fopen(filename, "rb");
    return f;
}

/**
 * open file for reading
 * safe version
 */
FILE* fileOpenReadSafe(char *filename){
    FILE* f;
    f = fileOpenRead(filename);
    if(!f)
        pError("can't open file for reading: %s",filename);
    return f;
}


/**
 * file close
 */
void fileClose(FILE* f){
    fclose(f);
}

/**
 * read the file
 */
int fileRead(FILE *f,void *buffer,int count){
    int read;
    read = fread(buffer,1,count,f);
    return read;
}

/**
 * read the file
 * safe version
 */
int fileReadSafe(FILE *f,void *buffer,int count){
    int read;
    read = fileRead(f,buffer,count);
    if(read != count)
        pError("file read failed");
    return read;
}


/**
 * write file
 */
int fileWrite(FILE *f,void *buffer,int count){
    int written;
    written = fwrite(buffer,1,count,f);
    return written;
}

/**
 * write file
 * safe version
 */
int fileWriteSafe(FILE *f,void *buffer,int count){
    int written;
    written = fileWrite(f,buffer,count);
    if(written != count)
        pError("file write failed");
    return written;
}

/**
 * load file into memory (the memory pointer MUST be freed later)
 */
int fileLoadSafe(char *filename,void **bufferptr){
    FILE    *f;
    int    length;
    void    *buffer;
    int     len;    // len that's a multiple of 512
    f = fileOpenReadSafe(filename);
    length = fileLength(f);
    len = (length / 512 + (length % 512 ? 1:0)) * 512;
    buffer = malloc(len);
    if(!buffer)
        pError("cannot allocate %d bytes of memory",length);
    fileReadSafe(f,buffer,length);
    fileClose(f);
    *bufferptr = buffer;
    return len;
}

/**
 * read input chunks, and write out the kernel image
 */
int main(int argc,char** argv){

    unsigned char *boot,*setup;
    int boot_len,setup_len;
    FILE *out;

    if(argc < 4)
        pError("usage: %s [boot][setup][out]",argv[0]);

    // load everything
    boot_len = fileLoadSafe(argv[1],(void**)&boot);
    setup_len = fileLoadSafe(argv[2],(void**)&setup);

    // prepare output
    out = fileOpenWriteSafe(argv[3]);

    // do some basic error checking (if we're still here)
    if(boot_len != 512)
        pError("boot secrot has to be exactly 512 bytes");

    // setup lengths
    boot[511] = setup_len / 512;

    // write out the image
    fileWriteSafe(out,boot,boot_len);
    fileWriteSafe(out,setup,setup_len);

    // close output
    fileClose(out);

    // free memory
    free(boot);
    free(setup);

    return 0;
}

